{ *************************************************************************** }
{                                                                             }
{ Snapshot for XPde                                                                    }
{                                                                             }
{ Copyright (c) 2004 Sergey Mishin <mishinmail@mail.ru>                       }
{                                                                             }
{ This program is free software; you can redistribute it and/or               }
{ modify it under the terms of the GNU General Public                         }
{ License as published by the Free Software Foundation; either                }
{ version 2 of the License, or (at your option) any later version.            }
{                                                                             }
{ This program is distributed in the hope that it will be useful,             }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of              }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           }
{ General Public License for more details.                                    }
{                                                                             }
{ You should have received a copy of the GNU General Public License           }
{ along with this program; see the file COPYING.  If not, write to            }
{ the Free Software Foundation, Inc., 59 Temple Place - Suite 330,            }
{ Boston, MA 02111-1307, USA.                                                 }
{                                                                             }
{ *************************************************************************** }

unit uSnapshot;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, QButtons, QComCtrls, uRegistry, XLib, Qt,
  StrUtils, libc;
                                            
type
  TSnapshotForm = class(TForm)
    GroupBox1: TGroupBox;
    Image: TImage;
    Timer: TTimer;
    btnGrab: TButton;
    btnClose: TButton;
    btnHelp: TButton;
    chkScreen: TCheckBox;
    Label1: TLabel;
    edtFilename: TEdit;
    sbFilename: TSpeedButton;
    btnSave: TButton;
    seDelay: TSpinEdit;
    Label2: TLabel;
    Shape: TShape;
    SaveDialog: TSaveDialog;
    procedure btnCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure btnGrabClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure SaveSettings();
    procedure sbFilenameClick(Sender: TObject);
    procedure seDelayChanged(Sender: TObject; NewValue: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  SnapshotForm: TSnapshotForm;
  Grabbing: boolean;
  Count: integer;
  Path : string;
  Screenshot: TBitmap;

implementation

{$R *.xfm}

procedure TSnapshotForm.SaveSettings();
    var reg: TRegistry;
begin
    reg := TRegistry.Create;

    try
        if reg.OpenKey('Software/XPde/snapshot', true) then begin
            reg.WriteInteger('Top', SnapshotForm.Top);
            reg.WriteInteger('Left', SnapshotForm.Left);
            reg.WriteInteger('Width', SnapshotForm.Width);
            reg.WriteInteger('Height', SnapshotForm.Height);
            reg.WriteInteger('Delay', seDelay.Value);
        end;
    finally
        reg.Free();
    end;
end;

procedure TSnapshotForm.btnCloseClick(Sender: TObject);
begin
     Close;
end;

procedure TSnapshotForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
     SaveSettings();
     Action:=caFree;
end;

procedure TSnapshotForm.FormCreate(Sender: TObject);
var reg: TRegistry;
    Root  : TWindow;
    Child : TWindow;
    Dummy : Integer;
    PPWRec  : PPasswordRecord;
    uid     : Integer;
begin

    font.name:='';
    parentfont:=true;

    reg := TRegistry.Create();
    try
        if Reg.OpenKey('Software/XPde/snapshot', false) then begin
            SnapshotForm.Top := reg.ReadInteger('Top');
            SnapshotForm.Left := reg.ReadInteger('Left');
            SnapshotForm.Height := reg.ReadInteger('Height');
            SnapshotForm.Width := reg.ReadInteger('Width');
            seDelay.Value:=reg.ReadInteger('Delay');
        end;
    finally
        reg.Free();
    end;

     Timer.Interval:=seDelay.Value*1000;
     Count:=1;

     uid := getuid;
     PPWRec := getpwuid(uid);
     if PPWRec^.pw_name ='root' then
         Path:='/'+PPWRec^.pw_name+'/'
     else Path:='/home/'+PPWRec^.pw_name+'/';

     with SaveDialog do
     begin
          InitialDir:=Path;
          DefaultExt:=GraphicExtension(TBitmap);
          Filter:=GraphicFilter(TBitmap);
     end;


     edtFilename.Text:=Path+'screen'+IntToStr(Count)+'.png';

     Screenshot := TBitmap.Create;
     XGrabPointer(QtDisplay,  QWidget_winID(Self.Handle), 1, ButtonPressMask,
                  GrabModeAsync, GrabModeAsync, 0, 0, 0);
     if XQueryPointer(QtDisplay, QWidget_winID(Application.Desktop),
                      @Root, @Child, @Dummy, @Dummy, @Dummy, @Dummy, @Dummy)<>0 then
     QPixmap_grabWindow(Screenshot.Handle, Root,0, 0, -1, -1);
     Image.Picture.Assign(Screenshot);


end;

procedure TSnapshotForm.btnGrabClick(Sender: TObject);
begin
     Grabbing := True;
     SnapshotForm.Hide;
     Timer.Enabled:=True;
end;

procedure TSnapshotForm.TimerTimer(Sender: TObject);
var
  Root  : TWindow;
  Child : TWindow;
  Dummy : Integer;
begin
      if Grabbing then
      begin
          Grabbing := False;
          if XQueryPointer(QtDisplay, QWidget_winID(Application.Desktop),
          @Root, @Child, @Dummy, @Dummy, @Dummy, @Dummy, @Dummy)<>0 then begin
                if chkScreen.Checked then begin
                    QPixmap_grabWindow(Screenshot.Handle, Child,0, 0, -1, -1);
                end
                else begin
                    QPixmap_grabWindow(Screenshot.Handle, Root,0, 0, -1, -1);
                end;
          end;
      end;
      Image.Picture.Assign(Screenshot);
      SnapshotForm.Show;
      //beep;
      Timer.Enabled:=False;
end;

procedure TSnapshotForm.btnHelpClick(Sender: TObject);
begin
     ShowMessage('Not implemented yet!');
end;

procedure TSnapshotForm.btnSaveClick(Sender: TObject);
begin
      
      Image.Picture.Bitmap.SaveToFile(edtFilename.Text);
      Inc(Count);
      edtFilename.Text:=Path+ 'screen'+ IntToStr(Count)+'.'+RightStr(edtFilename.Text,3);
end;

procedure TSnapshotForm.sbFilenameClick(Sender: TObject);
begin
     SaveDialog.InitialDir:=Path;

     if SaveDialog.Execute then begin
         edtFilename.Text:=SaveDialog.FileName;
         Path:=ExtractFilePath(SaveDialog.FileName);
     end;
end;

procedure TSnapshotForm.seDelayChanged(Sender: TObject; NewValue: Integer);
begin
     Timer.Interval:=seDelay.Value*1000;
end;

end.
